# 
# Exodus
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

class String
	
	# For miriam
	
	def is_miriam?
		return self =~ /^urn:miriam:/
	end
	
	def id
		return self.is_miriam? ? self.split(':', 4)[-1] : ''
	end
	
	def namespace
		return self.is_miriam? ? self.split(':', 4)[2] : ''
	end
	
	def resolve
		
	end
	
	# For HGV
	
	def is_hgvs?
		return self =~ Exodb::HGVPATTERN
	end
	
	def parse_hgvs
		
		result = {}
		
		Exodb::HGVPATTERN.match(self) do |m|
			
			if m[1] =~ /^chr/
				ref = m[1].split(/\./)
				result[:chr] = ref[0]
				result[:assembly] = ref[1].blank? ? Exodb::DEFAULTASSEMBLY : Exodb.assembly(ref[1])
			elsif m[1] =~ /^(\d{0,2}|[MXY])\./
				ref = m[1].split(/\./)
				result[:chr] = "chr#{ref[0]}"
				result[:assembly] = ref[1].blank? ? Exodb::DEFAULTASSEMBLY : Exodb.assembly(ref[1])
			else
				result[:chrrefseq] = m[1]
			end
			
			pos = m[2].split(/_/).sort
			result[:pos] = pos[0].to_i
			result[:start] = pos[0].to_i
			result[:stop] = pos[1].blank? ? pos[0].to_i : pos[1].to_i
			
			case m[3]
			when /^ins/
				result[:type] = 'ins'
				result[:alt] = m[3][3..-1]
			when /^del/
				result[:type] = 'del'
				result[:alt] = m[3][3..-1]
			else 
				result[:type] = 'sub'
				result[:alt] = m[3].split(/\>/)[1]
			end
			
		end
		
		return result
		
	end
	
	# For Pileup string
	def count_allele
		
		allellset = {
			'A' => "aA",
			'T' => "tT",
			'C' => "cC",
			'G' => "gG",
			'.' => "\\.\\,",
			'*' => "\\*"
		}
		
		tmpstr = self.dup
		
		allele = {}
		
		self.scan(/([+-])(\d+)([ATCGatcg]+)/) do |a, b, c|
			pattern = "#{a}#{b}#{c[0,(b.to_i)]}".upcase
			if !allele.has_key?(pattern)
				allele[pattern] = 0
				tmpstr.gsub!(/#{"#{b}#{c[0,(b.to_i)]}"}/, '')
			end
			allele[pattern] += 1
		end
		
		allellset.each_pair do |k, v|
			allele[k] = tmpstr.count(v) if tmpstr.count(v) > 0
		end
		
		return allele
		
	end
	
	# For pileup var
	def is_pileup_var?
		dat = self.split(/\//)
		return dat[0].is_loc? && dat[1] =~ /[\+\-]?[ATCG]+/
	end
	
	def parse_pileup_var
		
		result = {}
		if self.is_pileup_var?
			dat = self.split(/\//)
			result = dat[0].parse_loc
			dat[1] =~ /([\+\-]?)([ATCG]+)/
			result[:type] = case $1
			when '+'
				'ins'
			when '-'
				'del'
			else
				'sub'
			end
			result[:alt] = dat[1]
		end
		
		return result
		
	end
	
	# Location String
	
	def is_loc?
		return /^\w+:(\d+|\d+\.\.\d+|\d+-\d+)(:\w+)?$/
	end
	# For quality string
	
	# Assign gene location in format of chromosome_number:start..stop
	#
	# @param [String] gene location in format of chromosome_number:start..stop
	def parse_loc
		
		if self =~ /^[^:]+:(\d+|\d+\.\.\d+|\d+-\d+)(:\w+)?$/
			dat = self.split(/:/)
			pos = []
			dat[1].split(/\.\.|-/).each {|e| pos.push(e.to_i)}
			pos.sort!
			return {'chr' => dat[0], 'start' => pos[0], 'pos' => pos[0], 'stop' => pos[1] ? pos[1] : pos[0], 'assembly' => dat[2] ? Exodb::assembly(dat[2]) : Exodb::DEFAULTASSEMBLY}
		else
			raise
		end
		
		
	end
	
end