# 
# Exodb
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module Exodb
	
	class Reference
		
		include Mongoid::Document
		include Mongoid::Versioning
		include Mongoid::Timestamps
		
		include Exodb::XrefsField
		
		field :oid,					type: String
		
	end
	
	class Variantref < Reference
		
		PATTERN = /(?<gene>[A-Z0-9]+)-?(?<position>[0-9,]*|[is]?)(?<to>[A-Z=]*)/
		SILENTSIGN = '='
		
		include Exodb::VarLocationField
		
		field :reference,			type: String
		field :alternate,			type: String
		
	end
	
	class Occurrent
		
		include Mongoid::Document
		
		field :cancertype,			type: String
		field :position,			type: String # refseq id of chromomose
		field :occur,				type: Array
		field :casenumber,			type: Integer
		
		embedded_in :generef
		
	end
	
	class Mapping
		
		include Mongoid::Document
		
		field :chr,					type: String
		field :start,				type: Integer
		field :stop,				type: Integer
		field :tchr,				type: String
		field :tstart,				type: Integer
		field :tstop,				type: Integer
		field :coeff,				type: Integer # coefficient for conversion
		field :from,				type: String # from assembly version
		field :to,					type: String # to assembly version
		
		index({oid: 1, chr: 1, start: 1, stop: 1, from: 1, to: 1}, background: true)
		
		def self.convert(locstr, target = Exodb::LATESTASSEMBLY)
			begin
				query = parse_locstr(locstr)
				return self.where({from: query['assembly'], to: target, :start.lte => query[:pos], :stop.gte => query[:pos]}).first.convert(query[:pos])
			rescue
				
			end
		end
		
		def convert(pos)
			return  "#{self[:tchr]}:#{(coeff > 0 ? tstart : tstop) + ((pos - self[:start]) * coeff)}:#{to}"
		end
		
	end
	
end
