# 
# Exodb
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module Exodb
	
	class Source
		include Mongoid::Document
		#include Mongoid::Versioning
		include Mongoid::Timestamps
	end
	
	class Dataset < Source
		
		field :oid,					type: String
		field :name,				type: String
		field :experiment,			type: Array
		
		has_many :cells, autosave: true
		
		validates_uniqueness_of :oid, message: "Dataset oid of experiment is not unique"
		index({oid: 1}, {unique: false})
		index({name: 1})
	end
	
	class Cell < Source
		
		field :oid,					type: String
		field :patient,				type: String
		field :age,					type: String
		field :type,				type: String
		field :typeid,				type: String
		field :preferred,			type: Boolean
		field :paired,				type: Boolean
		field :purity,				type: Float
		field :labels,				type: Hash
		field :metastasis,			type: Boolean
		
		default_scope ->{where(preferred: true)}
		
		belongs_to :dataset
		has_many :variants, autosave: true, dependent: :delete
		has_many :genes, autosave: true, dependent: :delete
		
		validates_uniqueness_of :oid, message: "Cell sample oid of experiment is not unique"
		accepts_nested_attributes_for :variants
		
		index({oid: 1, type: 1, typeid: 1, paired: 1, purity: 1, preferred: 1})
		
		def self.translate!
			self.where({}).each {|e| e.translate!}
		end
		
		# get the start position of gene rely on the genome
		# 
		# @param [String] oid or 
		def add_to_dataset(str)
			
			dataset = Dataset.where('$or' => [{'oid' => str}, {'name' => str}])
			
			if dataset.exists?
				self.dataset = dataset.first()
				#output.puts "#EXODUS:INFO This sample is added to #{dataset.first().name}." if $0 == 'pry'
			else
				#output.puts "#EXODUS:ERRO Cannot find dataset by #{str}."  if $0 == 'pry'
			end
		end
		
		# Translate variant
		# 
		# @param [Float] cutoff score 
		def translate!(cutoff = 0)
			self.variants.each do |variant|
				
				Generef.cover?(variant.location_str).each do |generef|
					
					if generef.can_translated?
						
						mainsplice = generef.longest_splice
						position = mainsplice.get_prot_pos(variant.start)
						
						if !position.empty?
							gene = self.genes.find_or_create_by({symbol: generef.symbol})
							gene.generef = generef
							aacid = gene.aacids.find_or_create_by({position: /\A[A-Z]#{position[0]}[A-Z]?\z/ =~ variant.aachange ? position[0] : variant.aachange.split(/(\d+)/)[1].to_i})
							aacid.refcodon = mainsplice.get_codon(position[0]) if !aacid.refcodon
							aacid.refaa = mainsplice.get_codon(position[0]).translate if !aacid.refaa
							aacid.altcodon = {} if !aacid.altcodon
							aacid.altcodon[position[1]]  = aacid.altcodon[position[1]] ? aacid.altcodon[position[:posincodon]] | variant.alternate : variant.alternate
							aacid.isoform = [] if !aacid.isoform
							aacid.variants.push(variant)
							
							generef.splices.each do |splice|
								position = splice.get_prot_pos(variant.start)
								aacid.isoform.push("#{generef.get_xref()}:p.#{mainsplice.get_codon(position[0]).translate}#{position[0]}") if !position.empty?
							end
							gene.save!
						end
					end
				end
			end
		end
		
	end
	
end
