# 
# Exodus
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module Exodb

	class Variant
		
		include Mongoid::Document
		include Mongoid::Versioning
		include Mongoid::Timestamps
		
		include Exodb::VarLocationField
		
		#max_versions 5
		
		field :oid,					type: String # chromosome:position..alternative:samplename
		field :reference,			type: String #reference genotype
		field :genotypet,			type: Array # genotype from tumor cell
		field :genotypen,			type: Array # genotype from normal cell
		field :filter,				type: String
		field :passfilter,			type: Boolean #, default: false
		field :somstatus,			type: String #unknown, inherited, somatic
		field :somscore,			type: Float #Somatic score 0-1
		field :inhscore,			type: Float #Inheritant score 0-1
		#field :fdr,					type: Float #False discovery rate score 
		field :pileupt,				type: String #pileup string from tumor cell
		field :pileupn,				type: String #pileup string from normal cell
		field :qualt,				type: String #pileup quality from tumor cell
		field :qualn,				type: String #pileup quality from normal cell
		field :predicted_damage,	type: Boolean #Temporaly field
		field :evidence,			type: Array # experimental evidence of the variant
		field :temp,				type: Hash #Temporaly field
		
		belongs_to :cell
		belongs_to :aacid
		
		validates_uniqueness_of :oid, message: "Variant oid of experiment is not unique"
		
		index({oid: 1, reference: 1, alternatet: 1, passfilter: 1, somstatus: 1, somscore: 1, inhscore: 1, predicted_damage: 1}, background: true)
		
		before_save :update_oid
		
		# add this variant to original cell sample
		# 
		# @param [String] oid
		def add_to_sample(str)
			
			sample = Cell.where({'oid' => str})
			
			if sample.exists?
				self.cell = sample.first()
				#output.puts "#EXODB:INFO This sample is added to #{dataset.first().name}." if $0 == 'pry'
			else
				#output.puts "#EXODB:ERRO Cannot find dataset by #{str}." if $0 == 'pry'
			end
			
		end
		
		alias_method :add_to_cell, :add_to_sample
		
		
		# apply filter to the variant
		# the result is kelp in passfilter
		# 
		# @param [String] pile-up line from mpileup
		def apply_filter(filter)
			self[:filter] = filter
		end
		
		def alternate
			return self[:genotypet] - [self[:reference]]
		end
		
		protected
		
		def update_oid
			self.oid = "#{self.location_str}:#{self.location['assembly']}:#{self.cell.oid}"
		end
	end
	
end