# 
# Exodb
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module Exodb
    
	module VarLocationField
		
		extend ActiveSupport::Concern
		
		included do
			field :location,		type: Hash #{chr: '', position: x, assembly: x}
			field :convlocation,	type: Array
			
			validates_presence_of :location
			
			index({location: 1}, background: true)
			
		end
		
		module ClassMethods
			
			
		end
		
		# get the chromosome
		#
		# @return [Integer] chromosome
		def chromosome
			self[:location]['chr']
		end
		
		# Assign gene location in format of chromosome_number:pos;build
		#
		# @param [String] gene location in format of chromosome_number:start..stop
		def parse_locstr(loc_str)
			
			dat = loc_str.split(/:/)
			
			return {'chr' => dat[0],
				'position' => dat[1].to_i,
				'assembly' => dat[2] ? Exodb::ASSEMBLY.has_key?(dat[2]) ? Exodb::ASSEMBLY[dat[2]] : dat[2] : Exodb::DEFAULTASSEMBLY
			}
			
		end
		
		# Return location from specific genome assembly
		#
		# @param [String] assembly version
		#
		# @return [String] location string in chromosome:position
		def location_str(assembly = nil)
			
			result = nil
			
			if assembly == nil || Exodb::ASSEMBLY[assembly] == self[:location]['assembly']
				result = "#{self[:location]['chr']}:#{self[:location]['position']}:#{self[:location]['assembly']}"
			else
				self[:convlocation].each {|e| result = "#{[e['chr'], e['position'], e['assembly']].join(':')}" if e['assembly'] == Exodb::ASSEMBLY[assembly]}
			end
			
			return result
		end
		
		# Assign location
		#
		# @param [String, Hash] location string in chromosome:start..stop or chromosome:start-stop format
		def location=(loc)
			
			if loc.is_a?(String)
				
				begin
					self[:location] = parse_locstr(loc).delete_if {|k, v| ['start', 'stop'].include?(k)}
				rescue
					
				end
				
			end
		end
		
		# Assign location
		#
		# @param [String, Hash] location string in chromosome:start..stop or chromosome:start-stop format
		def convlocation=(loc)
			
			if loc.is_a?(String)
				
				begin
					self[:convlocation].push(parse_locstr(loc).delete_if {|k, v| ['start', 'stop'].include?(k)})
				rescue
					
				end
				
			end
			
		end
	end
    
end