# 
# Exodb
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module Exodb
	
	module_function
	
	# load session file
	#
	# @param [String] path to session file
    def sessionload!(sessionfile = nil)
		if sessionfile && File.exist?(sessionfile)
			Mongoid.load!(sessionfile, :production)
		else
			Mongoid.load!("#{Dir.pwd}/session.yml", :production) if File.exist?("#{Dir.pwd}/session.yml")
		end
    end
    
    # connect to the database without authentication
	#
	# @param [Hash, String] Connection string or a Hash in format { database: 'exodus', hosts: ['localhost:27017'], username: 'xxx', options: {}}
	def connect(connectionstr = {})
		
		settings = { database: 'exodus', hosts: ['localhost:27017'], options: {}}
		
		if connectionstr.is_a?(Hash)
			connectionstr.each_pair {|k, v| settings[k.to_sym] = k.to_sym == :hosts ? [v].flatten : v}
		elsif connectionstr.is_a?(String)
			split1 = connectionstr.split('@')
			settings[:username] = split1[0] if split1.length > 1
			split2 = split1[-1].split('/')
			settings[:database] = split2[1] if split2.length > 1
			settings[:hosts] = [split2[0]].flatten
		end
		
		password = ask("Password:  ") { |q| q.echo = "*" } if settings[:username] && !settings[:password]
		
		Mongoid::Sessions.disconnect
		Mongoid::Sessions.clear
		Mongoid.load_configuration({"sessions"=>{"default"=> password ? settings.merge({password: password}) : settings}})
		
		return "#EXODB:INFO Connection with #{settings}" if Pry.current
	end
	
	# connect to the database with authentication
	#
	# @param [String] (see #connect)
	#def connect!(db = 'exodus', hosts = ['localhost:27017'])
	#	username = ask("Username:  ")
	#	password = ask("Password:  ") { |q| q.echo = "*" }
	#	settings = {}
	#	if db
	#		settings = {"database"=>db, "hosts"=>[hosts].flatten, "username"=>username}
	#		Mongoid::Sessions.disconnect
	#		Mongoid::Sessions.clear
	#		Mongoid.load_configuration({"sessions"=>{"default"=>settings.merge({password: password})}})
	#	end
	#	return "#EXODUS:INFO Connection with #{settings} setting" if Pry.current
	#end
	
	# Return the session setting
	#
	# @return [String] the session setting
	def session
		Mongoid.session(:default)
	end
	
	# Return the current database name
	#
	# @return [String] the database name
	def current_database
		self.session.options[:database]
	end
	
end