# 
# Exodus
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"


module Exodb
	
	module Utils
		
		module_function
		
		# Upload gene information to database using gff3 and genome sequence fasta file
		# Exodb::Utils.upload_generef_from_gff3('ref_GRCh37.p5_top_level.gff3')
		#
		# @param [String] gff3 file
		# @param [String] assembly name [default: gff file name]
		def upload_generef_from_gff3(filename, assembly = Exodb::DEFAULTASSEMBLY)
			
			gff = Bio::GFF::GFF3.new(File.open(filename).read)
			
			processDbxref = lambda do |str|
				case str
				when /^GeneID/
					return "urn:miriam:ncbigene:#{str.split(/:/)[1]}"
				when /^HGNC/
					return "urn:miriam:hgnc:#{str}"
				when /^HPRD/
					return "urn:miriam:hprd:#{str.split(/:/)[1]}"
				when /^miRBase/
					return "urn:miriam:mirbase:#{str.split(/:/)[1]}"
				when /^Genbank/
					return "urn:miriam:refseq:#{str.split(/:/)[1]}"
				when /^CCDS/
					return "urn:miriam:ccds:#{str.split(/:/)[1]}"
				when /^MIM/
					return "urn:miriam:omim:#{str.split(/:/)[1]}"
				else
					return str
				end
			end
			
			assembly = assembly.blank? ? Exodb::DEFAULTASSEMBLY : Exodb::ASSEMBLY[assembly.downcase]
			
			regions = {}
			genes = {}
			seq = {}
			regions.default='chr0'
			
			gff.records.each do |e|
				
				case e.feature
				when 'region'
					e.attributes.each do |attr|
						case attr[0]
						when 'chromosome'
							regions[e.seqname] = e.seqname =~ /\ANC_/ ? "chr#{attr[1]}" : e.seqname
						end
					end
					
					chr = Exodb::Chrref.new()
					chr.location=("#{regions.has_key?(e.seqname) ? regions[e.seqname] : e.seqname}:#{e.start}..#{e.end}:#{assembly}")
					chr.oid = "#{chr.chr}:#{assembly}"
					chr.add_to_set(:xrefs, guess_miriam(e.seqname))
					chr.save!
					
					if File.exist?("./genome/#{e.seqname}.fa")
						seq = {}
						Bio::FlatFile.open(Bio::FastaFormat, "./genome/#{e.seqname}.fa").each {|fasta| seq[fasta.acc_version] = fasta.to_seq}
					end
					
				when 'gene', 'tRNA'
					
					gene = {type: 'gene', xrefs: [], strand: e.strand, chrrefseq: "#{guess_miriam(e.seqname)}", location: "#{regions[e.seqname]}:#{e.start}..#{e.end}:#{assembly}", childs: [], exon: [], cds: []}
					
					e.attributes.each do |attr|
						case attr[0]
						when 'Dbxref'
							gene[:xrefs].push(processDbxref.call(attr[1]))
						when 'Name'
							gene[:xrefs].push("urn:miriam:hgnc.symbol:#{attr[1]}") if attr[1] !~ /^LOC\d+$/
						when 'pseudo'
							gene[:psuedo] = attr[1] == 'true' ? true : false
						when 'ID'
							gene[:id] = attr[1]
						end
					end
					
					gene[:sequence] = seq[e.seqname].subseq(e.start.to_i - Exodb::Generef.expanding, e.end.to_i + Exodb::Generef.expanding).to_s if seq.has_key?(e.seqname)
					gene[:seqstart] = e.start.to_i - Exodb::Generef.expanding
					gene[:seqstop] = e.end.to_i + Exodb::Generef.expanding
					gene[:oid] = "#{gene[:location]}:#{assembly}"
					genes[gene[:id]] = gene
					
				when /\A(transcript|[^t]*RNA)/
					rna = {type: 'rna', xrefs: [], strand: e.strand, chr: regions[e.seqname], location: "#{regions[e.seqname]}:#{e.start}..#{e.end}:#{assembly}", exon: [], cds: []}
					
					e.attributes.each do |attr|
						case attr[0]
						when 'Dbxref'
							rna[:xrefs].push(processDbxref.call(attr[1]))
						when 'pseudo'
							rna[:psuedo] = attr[1] == 'true' ? true : false
						when 'ID'
							rna[:id] = attr[1]
						when 'Parent'
							rna[:parent] = attr[1]
						end
					end
					
					genes[rna[:id]] = rna
					genes[rna[:parent]][:childs].push(rna[:id]) if rna[:parent]
					
				when 'exon'
					e.attributes.each do |attr|
						case attr[0]
						when 'Parent'
							genes[attr[1]][:exon].push([e.start, e.end].sort)
						end
					end
				when 'CDS'
					e.attributes.each do |attr|
						case attr[0]
						when 'Parent'
							genes[attr[1]][:cds].push([e.start, e.end].sort)
						end
					end
				end
			end
			
			count = {succ: 0, fail: 0}
			
			genes.each_pair do |id, entry|
				if entry[:type] == 'gene'
					
					gene = Exodb::Generef.new()
					
					entry.each_pair do |k, v|
						gene.method(:"#{k}=").call(v) if ![:type, :childs, :exon, :cds].include?(k)
					end
					
					entry[:childs].each do |child|
						
						rna = Isoform.new()
						data = genes[child]
						rna.xrefs = data[:xrefs]
						rna.exon = data[:exon].sort
						rna.cds = data[:cds].sort
						
						gene.isoforms.push(rna)
						
					end
					
					if gene.save!
						count[:succ] += 1
						Exodb::putstv "Deposit Gene reference #{gene.xrefs[0]}"
					else
						count[:fail] += 1
						Exodb::putstv "Deposit Gene reference #{gene.xrefs[0]}"
					end
					
				end
				
			end
			
			Exodb::putst "SUCCESS: #{count[:succ]} , FAIL: #{count[:fail]}"
			
		end
	end
	
end