 
# Exodb
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

# Script used in Rositza project (Naevi, Spitzoid naevi, Spitzoid melanoma)

module Exodb
	
	module Utils
		
		module_function
		
		def load_snv_from_csv(csvfile)
			
			CSV.read(csvfile, col_sep: "\t", headers: true).each do |record|
				
				var = Exodb::Variant.new()
				var.location=("chr#{record["chromosome"]}:#{record["start position"]}")
				var.reference = record["ref nucleotide"].split('/')[0]
				var.alternatet = record["var nucleotide"].split('/').uniq
				var.somstatus = record["Somatic Status"].downcase
				var.passfilter = true
				var.pileupt = record["Reads"]
				var.predicted_damage = record["PolyPhen"] =~ /probably_damaging/ || record["SIFT"] =~ /deleterious/i || record["PROVEAN"] =~ /deleterious/i ? true : false
				var.temp = {} if var.temp.blank?
				var.temp['aachange'] = "#{record["symbol"]}:#{record["AA Change"]}"
				var.add_to_sample(record["cell"])
				
				p var.save!
			end
		end
		
		def load_splice_from_csv(csvfile)
			
			CSV.read(csvfile, col_sep: "\t", headers: true).each do |record|
				
				var = Exodb::Variant.new()
				var.location=("chr#{record["chr"]}:#{record["Position"]}")
				var.reference = record["Reference Genotype"]
				var.alternatet = ['N']
				var.somstatus = 'somatic'
				var.passfilter = true
				#var.pileupt = record["Reads"]
				var.predicted_damage = true
				var.temp = {} if var.temp.blank?
				var.temp['aachange'] = "#{record["Gene Symbol"]}:SpV"
				var.add_to_sample(record["Sample"])
				p var.save!
			end
		end
		
		def load_indel_from_csv(csvfile)
			
			CSV.read(csvfile, col_sep: "\t", headers: true).each do |record|
				
				var = Exodb::Variant.new()
				var.location=("#{record["chr"]}:#{record["pos"]}")
				#var.reference = record["Reference Genotype"]
				var.alternatet = ['-2N']
				var.somstatus = record["Somatic Status"].downcase
				var.passfilter = true
				#var.pileupt = record["Reads"]
				var.predicted_damage = true
				var.temp = {} if var.temp.blank?
				var.temp['aachange'] = "#{record["symbol"]}:InDel"
				var.add_to_sample(record["cell lines"])
				p var.save!
			end
		end
		
		def load_sample_from_csv(csvfile)
			CSV.read(csvfile, col_sep: "\t", headers: true).each do |record|
				
				sample = Cell.new({oid: record["SampleFinal"],
								type: record["Type"].downcase,
								typeid: "urn:miriam:bioportal.meddra:#{record["Type"] =~ /^spitz /i ? '10041632' : record["Type"] =~ /^spitzoid /i ? '10072450' : '10028679'}",
								patient: record["SampleFinal"].split('T')[0],
								preferred: record["Preferred"] == 'Y' ? true : false,
								paired: record["merge41final"] =~ /\Apaired\z/i ? true : false},
								labels: {})
				
				sample.add_to_dataset('internal.ds:000001')
				
				p sample.save!
				
			end
		end
		
		def load_variant_from_merge(mergefile, normal = false, assembly = Exodb::DEFAULTASSEMBLY)
			
			File.open(mergefile).each do |line|
				record = line.chomp.split("\t")
				
				header = record[0..15]
				
				list = record[16..-1]
				
				until list.empty?
					if normal
						snv = Exodb::Variant.where(oid: "#{header[2]}:#{header[3]}:#{assembly}:#{sampledata[0]}").first
						snv.pileupn = sampledata[6] if snv != nil
					else
						sampledata = list.shift(7)
						snv = Exodb::Variant.new()
						snv.location=("#{header[2]}:#{header[3]}:#{assembly}")
						snv.reference = header[10]
						snv.pileupt = sampledata[6]
						snv.temp = {} if snv.temp.blank?
						snv.add_to_sample(sampledata[0])
						p snv.save!
					end
				end
			end
			
		end
		
	end
end

#Exodb::Utils.load_sample_from_csv('Samples_all.txt')
#Exodb::Utils.load_snv_from_csv('NovelSNVs_13.txt')
#Exodb::Utils.load_variant_from_merge('gilead_ann/tumor/exome.UTR')
