# 
# Exodb
# Copyright (C) 2014
#
# @author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"


module Exodb
	
	module_function
	
	# General command for creating a user
	#
	# @param [String] user name to be created
	# @param [String] database that user created
	# @param [Array] roles to be assigned to the user
	def create_user(username, database, *roles)
		
		if database != 'admin'
			
			password = nil
			confirmed = nil
			
			while !password || password != confirmed
				password = ask("New password:  ") { |q| q.echo = "*" }
				confirmed = ask("Confirm password:  ") { |q| q.echo = "*" }
				
				puts "Password not match!\n" if password != confirmed
				
			end
			
			if database
				Mongoid.session(:default).with(database: database).command(
					createUser: username,
					pwd: password,
					roles: roles.flatten
				)
			else
				Mongoid.session(:default).command(
					createUser: username,
					pwd: password,
					roles: roles.flatten
				)
			end
		else
			raise CreateUserError, 'Cannot create user on admin database'
		end
	end
	
	# To create an admin user
	#
	# @param [String] user name to be created
	# @param [String] database that user created
	def create_admin(username, database)
		
		create_user(username, database, "readWrite", "dbAdmin", "userAdmin")
		
	end
	
	# To create a rw user
	#
	# @param [String] user name to be created
	# @param [String] database that user created
	def create_rwuser(username)
		
		create_user(username, Mongoid.session(:default).options[:database], "readWrite")
		
	end
	
	# To create a read-only user
	#
	# @param (see #create_rwuser)
	def create_rouser(username)
		
		create_user(username, Mongoid.session(:default).options[:database], "read")
		
	end
	
end