# 
# Exodb
# Copyright (C) 2014
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module Exodb
	
	module Ensembl
		
		module REST
			@@url = URI.parse('http://rest.ensembl.org')
			@@http = Net::HTTP.new(@@url.host, @@url.port)
			
			module_function
			
			def get(get_path, options)
				request = Net::HTTP::Get.new(get_path, {'Content-Type' => 'application/json'}.merge(options))
				response = @@http.request(request)
				if response.code != "200"
					riase InvalidResponse, "Invalid response: #{response.code}"
				else
					return JSON.parse(response.body)
				end
			end
			
			
			# http://rest.ensembl.org/documentation/info/vep_region_get
			def vep_region_get(region, allele, species, options = {})
				return Exodb::Ensembl::REST.get("vep/#{species}/region/#{region}/allele", options)
			end
			
			# http://rest.ensembl.org/documentation/info/vep_hgvs_get
			def vep_hgvs_get(hgvs, species, options = {})
				return Exodb::Ensembl::REST.get("vep/#{species}/hgvs/#{region}/allele", options)
			end
			
			# Converse coordinate from one asembly to another
			# from http://rest.ensembl.org/documentation/info/assembly_map
			#
			# @param [String] location string in chromosome:start..stop format
			# @param [String] target assembly or asm_two
			# @param [String] original assembly default: exodb::DEFAULTASSEMBLY
			# @param [String] species of genome
			# @param [Hash] option 
			def assembly_map(region, asm_two, asm_one = Exodb::DEFAULTASSEMBLY, species = 'human', options = {})
				return Exodb::Ensembl::REST.get("map/#{species.gsub(/\s+/, '_')}/#{Exodb::ASSEMBLY[asm_one.downcase]}/#{region}/#{Exodb::ASSEMBLY[asm_two.downcase]}", options)
			end
			
			# Returns the genomic sequence of the specified region of the given species.
			# from http://rest.ensembl.org/documentation/info/sequence_region
			#
			# @param [String] location string in chromosome:start..stop format
			# @param [String] species of genome
			# @param [Fasta] Return sequence 
			def sequence_region(region, species, options = {})
				return Exodb::Ensembl::REST.get("sequence/region/#{species}/#{region}", {'Content-Type' => 'text/x-fasta', 'coord_system_version' => Exodb::DEFAULTASSEMBLY}.merge(options))
			end
		end
		
	end
end