

class VCF
	
	attr_reader :info_attr, :filter_attr, :format_attr, :meta, :contig, :samples, :variants, :alt_attr
	
	def initialize()
		@info_attr = {}
		@filter_attr = {}
		@format_attr = {}
		@alt_attr = {}
		@meta = {}
		@contig = {}
		@variants = []

	end
	
	def open(filename)
		
		File.open(filename).each do |line|
			case line
			when /^##INFO/
				parse_info(line.chomp)
			when /^##FORMAT/
				parse_format(line.chomp)
			when /^##FILTER/
				parse_filter(line.chomp)
			when /^##contig/
				parse_contig(line.chomp)
			when /^##ALT/
				parse_alt(line.chomp)
			when /^##/
				parse_meta(line.chomp)
			when /^#/
				@samples = line.split(/\s+/)[9..-1]
			else
				@variants.push(Variant.new(line.chomp))
			end
			
		end
		
	end
	
	def fileformat
		return @meta[:fileformat]
	end
	
	def parse(line = 4000)
		@variants.each_index do |ind|
			break if ind > line
			Thread.new (ind) {
				@variants[ind].parse!(@info_attr, @format_attr)
			}
		end
	end
	
	def sample_num
		return @samples.length
	end
	
	def variant_num
		return @variants.length
	end
	
	class Variant
		
		attr_reader :chrom, :pos, :id, :ref, :alt, :qual, :filter, :info, :sample_data, :str
		
		CASTING = lambda do |string, type|
			case type
			when 'Integer'
				return string.to_i
			when 'Float'
				return string.to_f
			else
				return string.to_s
			end
		end
		
		def initialize(str)
			@str = str
		end
		
		def parse!(info_attr, format_attr)
			
			if @str
				splited = str.split(/\s+/)
				@chrom = splited[0]
				@pos = splited[1].to_i
				@id = splited[2]
				@ref = splited[3]
				@alt = splited[4]
				@qual = splited[5]
				@filter = splited[6]
				@info = {}
				parse_info(splited[7], info_attr)
				
				@formats = splited[8].split(/:/)
				@sample_data = []
				splited[9..-1].each {|data| @sample_data.push(parse_format(data, @formats, format_attr))}
				@str = nil
			end
			
		end
		
		private
		
		def parse_info(str, info_attr)
			str.split(/;/).each do |entry|
				splited = entry.split(/=/)
				case info_attr[splited[0]][:Number]
				when '1'
					@info[splited[0]] = CASTING.call(splited[1], info_attr[splited[0]][:Type])
				when '0'
					@info[splited[0]] = true
				else
					@info[splited[0]] = []
					splited[1].split(/,/).each {|e| @info[splited[0]].push(CASTING.call(e, info_attr[splited[0]][:Type]))}
				end
			end
		end
		
		def parse_format(str, formats, format_attr)
			
			result = {}
			
			splited = str.split(/:/)
			formats.each_index do |ind|
				case format_attr[formats[ind]][:Number]
				when '1'
					result[formats[ind]] = CASTING.call(splited[ind], format_attr[formats[0]][:Type])
				when '0'
					result[formats[ind]] = true
				else
					result[formats[ind]] = []
					formats[ind].split(/,/).each {|e| result[formats[ind]].push(CASTING.call(e, format_attr[formats[ind]][:Type]))}
				end
			end
			
			return result
		end
	end
	
	private
	
	def parse_meta(str)
		splited = str[2..-1].split(/=/)
		@meta[splited[0].to_sym] = splited[1]
	end
	
	def parse_contig(str)
		@contig = parse_data(str[10..-1])
	end
	
	def parse_filter(str)
		data = parse_data(str[10..-1])
		@filter_attr[data[:ID]] = data
	end
	
	def parse_info(str)
		data = parse_data(str[8..-1])
		@info_attr[data[:ID]] = data
	end
	
	def parse_format(str)
		data = parse_data(str[10..-1])
		@format_attr[data[:ID]] = data
	end
	
	def parse_alt(str)
		data = parse_data(str[7..-1])
		@alt_attr[data[:ID]] = data
	end
	
	def parse_data(str)
		result = {}
		str.scan(/(\w+=\w+|\w+=\".+\")/).each do |entry|
			splited = entry[0].split(/=/)
			result[splited[0].to_sym] = splited [1]
		end
		return result
	end
	
end


	
vcf = VCF.new()
vcf.open('./sample.vcf')
#p vcf.sample_num
#p vcf.variant_num
vcf.parse
p vcf